#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "lcd.h"

volatile uint8_t bitNumber = 0;
volatile uint16_t packet = 0;
volatile uint8_t newPacket = 0;
volatile uint8_t firstHalf = 0;

int main(void) {
	//podcignicie dla PD6
	PORTD = _BV(PD6);
	//preskaler 256, taktowanie 62500 Hz, tryb CTC
	TCCR1B = _BV(CS12) | _BV(WGM12);
	//reset timera co ok. 50 ms
	OCR1A = 3125;
	//przerwania dla input capture i output compare
	TIMSK = _BV(TICIE1) | _BV(OCIE1A);
	sei();
	lcdInit();
	lcdInitPrintf();
	set_sleep_mode(SLEEP_MODE_IDLE);
	sleep_enable();
    while (1) {
		sleep_cpu();
		if(newPacket){
			newPacket = 0;
			uint8_t command = packet & 0b00111111;
			uint8_t address = (packet >> 6) & 0b00011111;
			uint8_t toggle = (packet >> 11) & 1;
			lcdWriteCommand(LCD_COMMAND_CLEAR);
			printf("togg=%d address=%d", toggle, address);
			lcdGotoXY(0, 1);
			printf("command=%d", command);
		}
    }
}

ISR(TIMER1_CAPT_vect) {
	TCNT1 = 0;
	if (bitNumber) {
		//jeli mino mniej ni 83 cykle timera (1,3 ms)
		//czyli 3/4 bitu, wwczas jestemy
		//w przeciwnej powce bitu
		if (ICR1 < 83) firstHalf = !firstHalf;
	}
	if (!firstHalf) { //rodek bitu
		packet <<= 1; //przesu w lewo
		if (!(TCCR1B & _BV(ICES1))) { //zbocze opadajce
			packet |= 1; //wstaw jedynk na najmodszej pozycji
		}
		bitNumber++;
		if (bitNumber == 14) newPacket = 1;
	}
	//przestaw wykrywane zbocze na przeciwne
	TCCR1B ^= _BV(ICES1);
}

ISR(TIMER1_COMPA_vect) {
	TCCR1B &= ~_BV(ICES1);
	bitNumber = 0;
	packet = 0;
	firstHalf = 0;
}
